<?php
 /*
 Plugin Name: Easy Digital Downloads - Order Notes
 Plugin URI: https://freshlythemes.com/downloads/easy-digital-downloads-order-notes/
 Description: Capture special order instructions from your customers at the checkout.
 Version: 1.0.0
 Author: Freshly Themes
 Author URI: https://freshlythemes.com
 License: GPL-2.0+
 License URI: http://www.opensource.org/licenses/gpl-license.php
 Text Domain: edd-order-notes
 Domain Path: languages
 */

 // Exit if accessed directly
 if ( ! defined( 'ABSPATH' ) ) exit;


// Load plugin language file(s) or use fallback text domain
add_action( 'init', 'edd_order_notes_textdomain' );

// Check if Easy Digital Downloads is active
add_action( 'admin_init', 'edd_order_notes_check_edd' );

// Add order notes field to Easy Digital Downloads Checkout
add_action( 'edd_purchase_form_user_info_fields', 'edd_order_notes_check_downloads_as_services' );
add_filter( 'edd_payment_meta', 'edd_store_order_notes' );

// Add a metabox to the View Order Details screen within the EDD admin panel
add_action( 'edd_view_order_details_billing_after', 'edd_order_notes_view_order_details', 10, 1 );
add_action( 'edd_updated_edited_purchase', 'edd_order_notes_updated_edited_purchase', 10, 1 );

// Display order notes data on the Easy Digital Downloads payment receipt after the table
add_action( 'edd_payment_receipt_after_table', 'edd_order_notes_payment_receipt_after_table' );
add_action( 'fes_payment_receipt_after_table', 'edd_order_notes_payment_receipt_after_table' );

// Add order note data to Easy Digital Downloads payment history CSV export
add_action( 'edd_export_csv_cols_payments', 'edd_order_notes_payment_history_cols', 10 );
add_action( 'edd_export_get_data_payments', 'edd_order_notes_payment_history_data', 10 );


// Display order notes data within the Easy Digital Downloads payment receipt table
// add_action( 'edd_payment_receipt_after', 'edd_order_notes_payment_receipt_after' );
// add_action( 'fes_payment_receipt_after', 'edd_order_notes_payment_receipt_after' );

// Force the order notes field to be requirement at the checkout
// add_filter( 'edd_purchase_form_required_fields', 'edd_order_notes_purchase_form_required_fields' );

// Validate the order notes string length (140 character limit) before proceeding
// add_action( 'edd_checkout_error_checks', 'edd_order_notes_validate_checkout_fields', 10, 2 );


function edd_order_notes_textdomain() {
  // Set filter for plugin's languages directory
  $lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
  $lang_dir = apply_filters( 'edd_order_notes_lang_directory', $lang_dir );

  // Traditional WordPress plugin locale filter
  $locale        = apply_filters( 'plugin_locale',  get_locale(), 'edd-order-notes' );
  $mofile        = sprintf( '%1$s-%2$s.mo', 'edd-order-notes', $locale );

  // Setup paths to current locale file
  $mofile_local  = $lang_dir . $mofile;
  $mofile_global = WP_LANG_DIR . '/edd-order-notes/' . $mofile;

  if ( file_exists( $mofile_global ) ) {
    // Look in global /wp-content/languages/edd folder
    load_textdomain( 'edd-order-notes', $mofile_global );
  } elseif ( file_exists( $mofile_local ) ) {
    // Look in local /wp-content/plugins/easy-digital-downloads/languages/ folder
    load_textdomain( 'edd-order-notes', $mofile_local );
  } else {
    // Load the default language files
    load_plugin_textdomain( 'edd-order-notes', false, $lang_dir );
  }

}


function edd_order_notes_check_edd() {

	if ( ! class_exists( 'Easy_Digital_Downloads') ) {

		if ( is_plugin_active( plugin_basename( __FILE__ ) ) ) {

			deactivate_plugins( plugin_basename( __FILE__ ) );

	 		unset( $_GET[ 'activate' ] );

			add_action( 'admin_notices', 'edd_order_notes_admin_notice' );

		}

	}

}


function edd_order_notes_admin_notice() {

	ob_start();

	?>
	<div class="error">
		<p><?php _e( 'Order Notes requires Easy Digital Downloads! Please activate it to continue!', 'edd-order-notes' ); ?></p>
	</div>
	<?php

	echo ob_get_clean();

}


function edd_order_notes_check_downloads_as_services() {

  if ( class_exists( 'EDD_Downloads_As_Services' ) ) {
    edd_order_notes_get_cart_contents();
  } else {
    // add_filter( 'edd_payment_meta', 'edd_store_order_notes' );
    edd_output_order_notes();
  }

}


function edd_output_order_notes() {

  ?>
    <p id="edd-order-notes-wrap">
      <label class="edd-label" for="edd-message"><?php esc_html_e('Order Notes', 'edd-order-notes'); ?>
        <?php if( edd_field_is_required( 'edd_order_notes' ) ) { ?>
  			<span class="edd-required-indicator">*</span>
        <?php } ?>
      </label>
      <span class="edd-description" id="edd-order-notes-description"><?php esc_html_e('We will use this to customize your purchase.', 'edd-order-notes'); ?></span>
      <textarea class="edd-input required" name="edd_order_notes" placeholder="<?php esc_html_e( 'Please enter any special order instructions here.', 'edd-order-notes' ); ?>" id="edd-order-notes" <?php if( edd_field_is_required( 'edd_order_notes' ) ) {  echo ' required '; } ?> aria-describedby="edd-order-notes-description" /></textarea>
  </p>

  <?php

}


function edd_store_order_notes( $payment_meta ) {

	$payment_meta['order_notes'] = sanitize_textarea_field(isset( $_POST['edd_order_notes'] )) ? sanitize_textarea_field( $_POST['edd_order_notes'] ) : '';

	return $payment_meta;

}


function edd_order_notes_purchase_form_required_fields( $required_fields ) {

  $required_fields['edd_order_notes'] = array(
      'error_id' => 'invalid_order_notes',
      'error_message' => __( 'Please enter order notes', 'edd-order-notes' )
  );

  return $required_fields;

}


function edd_order_notes_validate_checkout_fields( $valid_data, $data ) {

    if ( strlen( $data['edd_order_notes'] ) > 140 ) {
      edd_set_error( 'invalid_order_notes', 'Order notes must be 140 characters or less.' );
    }

}


if (function_exists('edd_add_email_tag')) {

	edd_add_email_tag('order_notes', __( 'Special instructions submitted by the customer at the checkout.', 'edd-order-notes' ), 'edd_order_notes_email_tag' );

}


function edd_order_notes_email_tag( $payment_id ) {

	$payment_meta = edd_get_payment_meta( $payment_id );

  return $payment_meta['order_notes'];

}


function edd_order_notes_view_order_details( $payment_id ) {

	$payment_meta = edd_get_payment_meta( $payment_id );

	$order_notes = isset( $payment_meta['order_notes'] ) ? $payment_meta['order_notes'] : 'none'; ?>

  <div id="edd-payment-order-notes" class="postbox">
    <h3 class="hndle"><span><?php _e('Order Notes', 'edd-order-notes'); ?></span></h3>
    <div class="inside">
        <textarea name="edd-payment-order-notes" id="edd-payment-order-notes" class="large-text" style="width:100%; margin-top: 8px;"><?php esc_attr_e( $order_notes ); ?></textarea>
        <?php wp_nonce_field( plugin_basename( __FILE__ ), 'edd_order_notes_nonce' ); ?>
      <div class="clear"></div>
    </div><!-- /.inside -->
  </div><!-- /#edd-payment-order-notes -->

<?php
}


function edd_order_notes_updated_edited_purchase( $payment_id ) {

  // Check if the user intended to change this value.
  if ( ! isset( $_POST['edd_order_notes_nonce'] ) || ! wp_verify_nonce( $_POST['edd_order_notes_nonce'], plugin_basename( __FILE__ ) ) )
    return $payment_id;

	$payment_meta = edd_get_payment_meta( $payment_id );

	$payment_meta['order_notes'] = sanitize_textarea_field( isset( $_POST['edd-payment-order-notes'] ) ) ? sanitize_textarea_field( $_POST['edd-payment-order-notes'] ) : false;

	update_post_meta( $payment_id, '_edd_payment_meta', $payment_meta );

}


function edd_order_notes_payment_receipt_after_table( $payment ) {

	if (empty( $payment ) || empty( $payment->ID )) {
    return;
  }

	$payment_meta = edd_get_payment_meta( $payment->ID );

	$order_notes = isset( $payment_meta['order_notes'] ) ? $payment_meta['order_notes'] : 'none'; ?>

  <?php if (!empty( $order_notes )) { ?>
  	<h3><?php esc_html_e('Order Notes', 'edd-order-notes'); ?></h3>
    <div class="edd_purchase_receipt_order_notes"><?php echo wpautop( $order_notes ); ?></div>
  <?php } ?>

<?php
}


function edd_order_notes_payment_receipt_after( $payment ) {

	if (empty( $payment ) || empty( $payment->ID )) {
    return;
  }

	$payment_meta = edd_get_payment_meta( $payment->ID );

	$order_notes = isset( $payment_meta['order_notes'] ) ? $payment_meta['order_notes'] : 'none'; ?>

  <?php if (!empty( $order_notes )) { ?>

	<tr>
		<td><strong><?php _e( 'Order Notes', 'edd-order-notes' ); ?>:</strong></td>
		<td><?php echo wpautop( $order_notes ); ?></td>
	</tr>

  <?php } ?>

<?php
}

function edd_order_notes_payment_history_data( $data ) {

  foreach($data as $index => $payment) {

  	$payment_meta = edd_get_payment_meta( $payment['id'] );

    $order_notes = isset($payment_meta['order_notes']) ? $payment_meta['order_notes'] : 'none';

    $order_notes_stripped = preg_replace( "/\r|\n/", "", $order_notes );

    $data[$index]['order_notes'] = $order_notes_stripped;

  }

	return $data;

}


function edd_order_notes_payment_history_cols( $cols ) {

  global $edd_options;

	 $cols['order_notes'] = __('Order Notes', 'edd-order-notes');

	return $cols;

}

function edd_order_notes_get_cart_contents() {

  $cart_contents = edd_get_cart_contents();

  if ( ! empty( $cart_contents ) ) {

  foreach ( $cart_contents as $item ) {

      if ( edd_order_notes_is_service( $item['id'] ) ) {
          add_filter( 'edd_purchase_form_required_fields', 'edd_order_notes_purchase_form_required_fields' );
          edd_output_order_notes();
          return;
      }
    }
  }
}

function edd_order_notes_is_service( $item_id ) {

  global $edd_options;

	// get array of service categories
	$service_categories = isset( $edd_options['edd_das_service_categories'] ) ? $edd_options['edd_das_service_categories'] : '';

	$term_ids = array();

	if ( $service_categories ) {
		foreach ( $service_categories as $term_id => $term_name ) {
			$term_ids[] = $term_id;
		}
	}
	$is_service = get_post_meta( $item_id, '_edd_das_enabled', true );

	// check if download has meta key or has a service term assigned to it
	if ( $is_service || ( ! empty( $term_ids ) && has_term( $term_ids, 'download_category', $item_id ) ) ) {
		return true;
	}
	return false;

}

?>
